package Math::Geometry::Planar::GPC::Inherit;
$VERSION = 0.11;

require Exporter;
require DynaLoader;
@ISA = qw(Exporter DynaLoader);
bootstrap Math::Geometry::Planar::GPC::Inherit $Math::Geometry::Planar::GPC::Inherit::VERSION;

1;
